@@ -13,14 +13,31 @@ pr_title="Synchronize common files from prometheus/prometheus"
 pr_msg="Propagating changes from prometheus/prometheus default branch."
 orgs="prometheus prometheus-community"
 
+color_red='\e[31m'
+color_green='\e[32m'
+color_yellow='\e[33m'
+color_none='\e[0m'
+
+echo_red() {
+  echo -e "${color_red}$@${color_none}" 1>&2
+}
+
+echo_green() {
+  echo -e "${color_green}$@${color_none}" 1>&2
+}
+
+echo_yellow() {
+  echo -e "${color_yellow}$@${color_none}" 1>&2
+}
+
 GITHUB_TOKEN="${GITHUB_TOKEN:-}"
 if [ -z "${GITHUB_TOKEN}" ]; then
-  echo -e "\e[31mGitHub token (GITHUB_TOKEN) not set. Terminating.\e[0m"
+  echo_red 'GitHub token (GITHUB_TOKEN) not set. Terminating.'
   exit 1
 fi
 
 # List of files that should be synced.
-SYNC_FILES="CODE_OF_CONDUCT.md LICENSE Makefile.common SECURITY.md"
+SYNC_FILES="CODE_OF_CONDUCT.md LICENSE Makefile.common SECURITY.md .yamllint"
 
 # Go to the root of the repo
 cd "$(git rev-parse --show-cdup)" || exit 1
@@ -30,6 +47,13 @@ source_dir="$(pwd)"
 tmp_dir="$(mktemp -d)"
 trap 'rm -rf "${tmp_dir}"' EXIT
 
+prometheus_orb_version="$(yq eval '.orbs.prometheus' .circleci/config.yml)"
+if [[ -z "${prometheus_orb_version}" ]]; then
+  echo_red '"ERROR: Unable to get CircleCI orb version'
+  exit 1
+fi
+
+## Internal functions
 github_api() {
   local url
   url="https://api.github.com/${1}"
@@ -43,8 +67,8 @@ get_default_branch() {
 }
 
 fetch_repos() {
-  github_api "users/${1}/repos?per_page=100" 2> /dev/null |
-    jq -r '.[] | select( .name != "prometheus" ) | .name'
+  github_api "orgs/${1}/repos?type=public&per_page=100" 2> /dev/null |
+    jq -r '.[] | select( .archived == false and .fork == false and .name != "prometheus" ) | .name'
 }
 
 push_branch() {
@@ -72,11 +96,29 @@ check_license() {
   echo "$1" | grep --quiet --no-messages --ignore-case 'Apache License'
 }
 
+check_circleci_orb() {
+  local org_repo
+  local default_branch
+  org_repo="$1"
+  default_branch="$2"
+  local ci_config_url="https://raw.githubusercontent.com/${org_repo}/${default_branch}/.circleci/config.yml"
+
+  orb_version="$(curl -sL --fail "${ci_config_url}" | yq eval '.orbs.prometheus' -)"
+  if [[ -z "${orb_version}" ]]; then
+    echo_yellow "WARNING: Failed to fetch CirleCI orb version from '${org_repo}'"
+    return 0
+  fi
+
+  if [[ "${orb_version}" != "null" && "${orb_version}" != "${prometheus_orb_version}" ]]; then
+    echo "CircleCI-orb"
+  fi
+}
+
 process_repo() {
   local org_repo
   local default_branch
   org_repo="$1"
-  echo -e "\e[32mAnalyzing '${org_repo}'\e[0m"
+  echo_green "Analyzing '${org_repo}'"
 
   default_branch="$(get_default_branch "${org_repo}")"
   if [[ -z "${default_branch}" ]]; then
@@ -89,7 +131,7 @@ process_repo() {
   for source_file in ${SYNC_FILES}; do
     source_checksum="$(sha256sum "${source_dir}/${source_file}" | cut -d' ' -f1)"
 
-    target_file="$(curl -s --fail "https://raw.githubusercontent.com/${org_repo}/${default_branch}/${source_file}")"
+    target_file="$(curl -sL --fail "https://raw.githubusercontent.com/${org_repo}/${default_branch}/${source_file}")"
     if [[ "${source_file}" == 'LICENSE' ]] && ! check_license "${target_file}" ; then
       echo "LICENSE in ${org_repo} is not apache, skipping."
       continue
@@ -113,6 +155,13 @@ process_repo() {
     needs_update+=("${source_file}")
   done
 
+  local circleci
+  circleci="$(check_circleci_orb "${org_repo}" "${default_branch}")"
+  if [[ -n "${circleci}" ]]; then
+    echo "${circleci} needs updating."
+    needs_update+=("${circleci}")
+  fi
+
   if [[ "${#needs_update[@]}" -eq 0 ]] ; then
     echo "No files need sync."
     return
@@ -125,7 +174,10 @@ process_repo() {
 
   # Update the files in target repo by one from prometheus/prometheus.
   for source_file in "${needs_update[@]}"; do
-    cp -f "${source_dir}/${source_file}" "./${source_file}"
+    case "${source_file}" in
+      CircleCI-orb) yq eval -i ".orbs.prometheus = \"${prometheus_orb_version}\"" .circleci/config.yml ;;
+      *) cp -f "${source_dir}/${source_file}" "./${source_file}" ;;
+    esac
   done
 
   if [[ -n "$(git status --porcelain)" ]]; then
@@ -144,6 +196,7 @@ process_repo() {
   fi
 }
 
+## main
 for org in ${orgs}; do
   mkdir -p "${tmp_dir}/${org}"
   # Iterate over all repositories in ${org}. The GitHub API can return 100 items
@@ -154,13 +207,13 @@ for org in ${orgs}; do
     fetch_uri="repos/${org}/${repo}/pulls?state=open&head=${org}:${branch}"
     prLink="$(github_api "${fetch_uri}" --show-error | jq -r '.[0].html_url')"
     if [[ "${prLink}" != "null" ]]; then
-      echo "Pull request already opened for branch '${branch}': ${prLink}"
+      echo_green "Pull request already opened for branch '${branch}': ${prLink}"
       echo "Either close it or merge it before running this script again!"
       continue
     fi
 
     if ! process_repo "${org}/${repo}"; then
-      echo "Failed to process '${org}/${repo}'"
+      echo_red "Failed to process '${org}/${repo}'"
       exit 1
     fi
   done
